# coding=utf-8
from __future__ import division

from ladybug.datacollection import HourlyDiscontinuousCollection, \
    HourlyContinuousCollection, MonthlyCollection, DailyCollection, \
    MonthlyPerHourCollection
from ladybug.datacollectionimmutable import HourlyDiscontinuousCollectionImmutable, \
    HourlyContinuousCollectionImmutable, MonthlyCollectionImmutable, \
    DailyCollectionImmutable, MonthlyPerHourCollectionImmutable
from ladybug.header import Header
from ladybug.analysisperiod import AnalysisPeriod
from ladybug.dt import DateTime
from ladybug.datatype.temperature import Temperature
from ladybug.datatype.fraction import RelativeHumidity

import unittest
import pytest
import sys
if (sys.version_info >= (3, 0)):
    xrange = range


class DataCollectionTestCase(unittest.TestCase):
    """Test for (ladybug/datacollectionimmutable.py)"""

    def test_hourly(self):
        """Test the dicontinuous collections."""
        a_per = AnalysisPeriod(6, 21, 12, 6, 21, 13)
        dt1, dt2 = DateTime(6, 21, 12), DateTime(6, 21, 13)
        v1, v2 = 20, 25
        dc1 = HourlyDiscontinuousCollectionImmutable(
            Header(Temperature(), 'C', a_per), [v1, v2], [dt1, dt2])
        assert dc1.datetimes == (dt1, dt2)
        assert dc1.values == (v1, v2)
        assert dc1.is_mutable is False
        with pytest.raises(AttributeError):
            dc1[0] = 18
        with pytest.raises(AttributeError):
            dc1.values = [18, 24]
        with pytest.raises(Exception):
            dc1.values.append(10)
        with pytest.raises(AttributeError):
            dc2 = dc1.convert_to_culled_timestep(1)

        dc2 = dc1.to_mutable()
        assert isinstance(dc2, HourlyDiscontinuousCollection)
        assert dc2.is_mutable is True
        dc2[0] = 18
        assert dc2[0] == 18
        dc2.values = [18, 24]
        assert dc2.values == (18, 24)
        with pytest.raises(Exception):
            dc2.values.append(10)  # make sure that we can still not append

        dc3 = dc2.to_immutable()
        assert isinstance(dc3, HourlyDiscontinuousCollectionImmutable)
        assert dc3.is_mutable is False

        dc4 = dc3.to_immutable()
        assert isinstance(dc4, HourlyDiscontinuousCollectionImmutable)
        assert dc4.is_mutable is False

    def test_init_continuous(self):
        """Test the init methods for continuous collections"""
        # Setup temperature data collection
        header = Header(Temperature(), 'C', AnalysisPeriod())
        values = list(xrange(8760))
        dc1 = HourlyContinuousCollectionImmutable(header, values)
        assert len(dc1.datetimes) == 8760
        assert list(dc1.values) == list(xrange(8760))
        assert dc1.is_mutable is False
        with pytest.raises(AttributeError):
            dc1[0] = 18
        with pytest.raises(AttributeError):
            dc1.values = [24] * 8760
        with pytest.raises(Exception):
            dc1.values.append(10)
        with pytest.raises(AttributeError):
            dc2 = dc1.convert_to_culled_timestep(1)

        dc2 = dc1.to_mutable()
        assert isinstance(dc2, HourlyContinuousCollection)
        assert dc2.is_mutable is True
        dc2[0] = 18
        assert dc2[0] == 18
        dc2.values = [24] * 8760
        assert dc2.values == tuple([24] * 8760)
        with pytest.raises(Exception):
            dc2.values.append(10)  # make sure that we can still not append

        dc3 = dc2.to_immutable()
        assert isinstance(dc3, HourlyContinuousCollectionImmutable)
        assert dc3.is_mutable is False

        dc4 = dc3.to_immutable()
        assert isinstance(dc4, HourlyContinuousCollectionImmutable)
        assert dc4.is_mutable is False

    def test_daily(self):
        """Test the daily collections."""
        a_per = AnalysisPeriod(6, 21, 0, 6, 22, 23)
        v1, v2 = 20, 25
        dc1 = DailyCollectionImmutable(
            Header(Temperature(), 'C', a_per), [v1, v2], a_per.doys_int)
        assert dc1.datetimes == tuple(a_per.doys_int)
        assert dc1.values == (v1, v2)
        assert dc1.is_mutable is False
        with pytest.raises(AttributeError):
            dc1[0] = 18
        with pytest.raises(AttributeError):
            dc1.values = [18, 24]
        with pytest.raises(Exception):
            dc1.values.append(10)

        dc2 = dc1.to_mutable()
        assert isinstance(dc2, DailyCollection)
        assert dc2.is_mutable is True
        dc2[0] = 18
        assert dc2[0] == 18
        dc2.values = [18, 24]
        assert dc2.values == (18, 24)
        with pytest.raises(Exception):
            dc2.values.append(10)  # make sure that we can still not append

        dc3 = dc2.to_immutable()
        assert isinstance(dc3, DailyCollectionImmutable)
        assert dc3.is_mutable is False

        dc4 = dc3.to_immutable()
        assert isinstance(dc4, DailyCollectionImmutable)
        assert dc4.is_mutable is False

    def test_monthly(self):
        """Test the monthly collections."""
        a_per = AnalysisPeriod(6, 1, 0, 7, 31, 23)
        v1, v2 = 20, 25
        dc1 = MonthlyCollectionImmutable(
            Header(Temperature(), 'C', a_per), [v1, v2], a_per.months_int)

        assert dc1.datetimes == tuple(a_per.months_int)
        assert dc1.values == (v1, v2)
        assert dc1.is_mutable is False
        with pytest.raises(AttributeError):
            dc1[0] = 18
        with pytest.raises(AttributeError):
            dc1.values = [18, 24]
        with pytest.raises(Exception):
            dc1.values.append(10)

        dc2 = dc1.to_mutable()
        assert isinstance(dc2, MonthlyCollection)
        assert dc2.is_mutable is True
        dc2[0] = 18
        assert dc2[0] == 18
        dc2.values = [18, 24]
        assert dc2.values == (18, 24)
        with pytest.raises(Exception):
            dc2.values.append(10)  # make sure that we can still not append

        dc3 = dc2.to_immutable()
        assert isinstance(dc3, MonthlyCollectionImmutable)
        assert dc3.is_mutable is False

        dc4 = dc3.to_immutable()
        assert isinstance(dc4, MonthlyCollectionImmutable)
        assert dc4.is_mutable is False

    def test_monthly_per_hour(self):
        """Test the monthly per hour collections."""
        a_per = AnalysisPeriod(6, 1, 0, 7, 31, 23)
        vals = [20] * 24 + [25] * 24
        dc1 = MonthlyPerHourCollectionImmutable(
            Header(Temperature(), 'C', a_per), vals, a_per.months_per_hour)
        assert dc1.datetimes == tuple(a_per.months_per_hour)
        assert dc1.values == tuple(vals)
        assert dc1.is_mutable is False
        with pytest.raises(AttributeError):
            dc1[0] = 18
        with pytest.raises(AttributeError):
            dc1.values = range(48)
        with pytest.raises(Exception):
            dc1.values.append(10)

        dc2 = dc1.to_mutable()
        assert isinstance(dc2, MonthlyPerHourCollection)
        assert dc2.is_mutable is True
        dc2[0] = 18
        assert dc2[0] == 18
        dc2.values = range(48)
        assert dc2.values == tuple(range(48))
        with pytest.raises(Exception):
            dc2.values.append(10)  # make sure that we can still not append

        dc3 = dc2.to_immutable()
        assert isinstance(dc3, MonthlyPerHourCollectionImmutable)
        assert dc3.is_mutable is False

        dc4 = dc3.to_immutable()
        assert isinstance(dc4, MonthlyPerHourCollectionImmutable)
        assert dc4.is_mutable is False

    def test_get_aligned_collection(self):
        """Test the method for getting an aligned discontinuous collection."""
        header = Header(Temperature(), 'C', AnalysisPeriod(end_month=1, end_day=1))
        values = list(xrange(24))
        dc1 = HourlyDiscontinuousCollection(header, values,
                                            header.analysis_period.datetimes)
        dc2 = dc1.get_aligned_collection(50, RelativeHumidity(), '%')
        assert dc2.header.data_type.name == 'Relative Humidity'
        assert dc2.header.unit == '%'
        assert isinstance(dc2, HourlyDiscontinuousCollection)
        assert dc2.is_mutable is True

        dc3 = dc1.get_aligned_collection(50, RelativeHumidity(), '%', mutable=False)
        assert dc3.header.data_type.name == 'Relative Humidity'
        assert dc3.header.unit == '%'
        assert isinstance(dc3, HourlyDiscontinuousCollectionImmutable)
        assert dc3.is_mutable is False

        dc4 = dc1.get_aligned_collection(50, RelativeHumidity(), '%', mutable=True)
        assert dc4.header.data_type.name == 'Relative Humidity'
        assert dc4.header.unit == '%'
        assert isinstance(dc4, HourlyDiscontinuousCollection)
        assert dc4.is_mutable is True

    def test_get_aligned_collection_continuous(self):
        """Test the method for getting an aligned continuous collection."""
        header = Header(Temperature(), 'C', AnalysisPeriod(end_month=1, end_day=1))
        values = list(xrange(24))
        dc1 = HourlyContinuousCollection(header, values)
        dc2 = dc1.get_aligned_collection(50, RelativeHumidity(), '%')
        assert dc2.header.data_type.name == 'Relative Humidity'
        assert dc2.header.unit == '%'
        assert isinstance(dc2, HourlyContinuousCollection)
        assert dc2.is_mutable is True

        dc3 = dc1.get_aligned_collection(50, RelativeHumidity(), '%', mutable=False)
        assert dc3.header.data_type.name == 'Relative Humidity'
        assert dc3.header.unit == '%'
        assert isinstance(dc3, HourlyContinuousCollectionImmutable)
        assert dc3.is_mutable is False

        dc4 = dc1.get_aligned_collection(50, RelativeHumidity(), '%', mutable=True)
        assert dc4.header.data_type.name == 'Relative Humidity'
        assert dc4.header.unit == '%'
        assert isinstance(dc4, HourlyContinuousCollection)
        assert dc4.is_mutable is True


if __name__ == "__main__":
    unittest.main()



# coding=utf-8

from ladybug.analysisperiod import AnalysisPeriod
from ladybug.dt import DateTime

from datetime import timedelta
import unittest
import sys
if (sys.version_info >= (3, 0)):
    xrange = range


class AnalysisPeriodTestCase(unittest.TestCase):
    """Test for (analysisperiod.py)"""

    def test_default_values(self):
        """Test the default values."""
        ap = AnalysisPeriod()
        assert ap.st_hour == 0
        assert ap.end_hour == 23
        assert ap.st_month == 1
        assert ap.end_month == 12
        assert ap.st_day == 1
        assert ap.end_day == 31
        assert ap.timestep == 1
        assert ap.is_leap_year is False
        assert len(ap.datetimes) == 8760

    def test_from_string(self):
        """Test creating analysis priod from a string."""
        ap_string = '2/21 to 3/22 between 5 to 17 @1'
        ap = AnalysisPeriod.from_analysis_period(ap_string)
        ap = AnalysisPeriod.from_string(ap_string)
        assert ap.st_hour == 5
        assert ap.end_hour == 17
        assert ap.st_month == 2
        assert ap.end_month == 3
        assert ap.st_day == 21
        assert ap.end_day == 22
        assert ap.timestep == 1

    def test_is_annual(self):
        """Test the is_annual property."""
        annual_ap = AnalysisPeriod()
        not_annual_ap = AnalysisPeriod(end_month=2)
        assert annual_ap.is_annual
        assert not not_annual_ap.is_annual

    def test_to_from_json(self):
        """Test the json methods of the AnalysisPeriod."""
        ap = AnalysisPeriod(st_month=2)
        ap_json = ap.to_json()
        rebuilt_ap = AnalysisPeriod.from_json(ap_json)
        assert rebuilt_ap.st_month == 2
        assert rebuilt_ap.to_json() == ap_json

    def test_include_last_hour(self):
        """Test that analysis period includes the last hour.

        see:
        https://github.com/ladybug-tools/ladybug-dynamo/issues/12#issuecomment-224125154
        """
        ap = AnalysisPeriod(7, 21, 9, 7, 21, 15, 4)
        last_hour = ap.datetimes[-1]
        assert last_hour.hour == 15
        assert last_hour.minute == 0
        assert len(ap) == 25

    def test_default_values_leap_year(self):
        """Test the default values."""
        ap = AnalysisPeriod(is_leap_year=True)
        assert ap.st_hour == 0
        assert ap.end_hour == 23
        assert ap.st_month == 1
        assert ap.end_month == 12
        assert ap.st_day == 1
        assert ap.end_day == 31
        assert ap.timestep == 1
        assert len(ap.datetimes) == 8784

    def test_from_string_leap_year(self):
        """Test creating analysis priod from a string."""
        ap_string = '2/21 to 3/22 between 5 to 17 @1*'
        ap = AnalysisPeriod.from_string(ap_string)
        assert ap.st_hour == 5
        assert ap.end_hour == 17
        assert ap.st_month == 2
        assert ap.end_month == 3
        assert ap.st_day == 21
        assert ap.end_day == 22
        assert ap.timestep == 1

    def test_is_annual_leap_year(self):
        annual_ap = AnalysisPeriod(is_leap_year=True)
        not_annual_ap = AnalysisPeriod(end_month=2, is_leap_year=True)
        assert annual_ap.is_annual
        assert not not_annual_ap.is_annual

    def test_moys_hoys_hoys_int(self):
        """Test the moys, hoys, and hoys_int properties."""
        ap = AnalysisPeriod(timestep=4)
        assert ap.moys[1] == 15
        assert ap.hoys[1] == 0.25
        assert ap.hoys_int[1] == 0
        assert isinstance(ap.hoys[1], float)
        assert isinstance(ap.hoys_int[1], int)
        assert len(ap.moys) == len(ap.hoys) == len(ap.hoys_int)

    def test_doys_int(self):
        """Test the doys_int property."""
        ap = AnalysisPeriod()
        ap_2 = AnalysisPeriod(st_month=2)
        assert ap.doys_int == list(xrange(1, 366))
        assert ap_2.doys_int == list(xrange(32, 366))

    def test_months_int(self):
        """Test the months_int property."""
        ap = AnalysisPeriod()
        ap_2 = AnalysisPeriod(st_month=2)
        assert ap.months_int == list(xrange(1, 13))
        assert ap_2.months_int == list(xrange(2, 13))

    def test_months_per_hour(self):
        """Test the months_per_hour property."""
        ap = AnalysisPeriod()
        ap_2 = AnalysisPeriod(st_month=2)
        ap_3 = AnalysisPeriod(st_hour=5)
        assert len(ap.months_per_hour) == 12 * 24
        assert len(ap_2.months_per_hour) == 11 * 24
        assert len(ap_3.months_per_hour) == 12 * 19

    def test_minute_intervals(self):
        """Test the minute_intervals property."""
        ap = AnalysisPeriod()
        ap_2 = AnalysisPeriod(timestep=6)
        assert ap.minute_intervals == timedelta(minutes=60)
        assert ap_2.minute_intervals == timedelta(minutes=10)

    def test_is_reversed(self):
        """Test the is_reversed property."""
        ap = AnalysisPeriod()
        ap_2 = AnalysisPeriod(st_month=6, end_month=2)
        assert not ap.is_reversed
        assert ap_2.is_reversed

    def test_is_overnight(self):
        """Test the is_overnight property."""
        ap = AnalysisPeriod()
        ap_2 = AnalysisPeriod(st_hour=20, end_hour=5)
        assert not ap.is_overnight
        assert ap_2.is_overnight

    def test_is_time_included(self):
        """Test the is_time_included method."""
        ap = AnalysisPeriod(st_month=2, end_month=6)
        assert ap.is_time_included(DateTime(5, 18, 17))
        assert not ap.is_time_included(DateTime(1, 10, 12))

    def test_duplicate(self):
        """Test the duplicate method."""
        ap = AnalysisPeriod()
        ap_2 = ap.duplicate()
        assert ap_2 == ap

    def test_0_end_hour(self):
        """Test hour 0 for end hour."""
        params = [1, 1, 1, 1, 2, 0]
        ap = AnalysisPeriod(*params)
        assert ap.end_hour == 0
        assert len(ap) == 24

if __name__ == "__main__":
    unittest.main()



# coding=utf-8

import unittest
from pytest import approx
import os
from ladybug.location import Location
from ladybug.analysisperiod import AnalysisPeriod
from ladybug.designday import DDY, DesignDay


class DdyTestCase(unittest.TestCase):
    """Test for (ladybug/designday.py)"""

    def test_import_ddy(self):
        """Test import standard ddy."""
        relative_path = './tests/ddy/chicago.ddy'
        abs_path = os.path.abspath(relative_path)

        ddy_rel = DDY.from_ddy_file(relative_path)
        ddy = DDY.from_ddy_file(abs_path)

        # Test imports don't break
        assert ddy.file_path == abs_path
        assert ddy_rel.file_path == os.path.normpath(relative_path)

    def test_json_methods(self):
        """Test json methods for the DDY object."""
        relative_path = './tests/ddy/chicago.ddy'
        ddy = DDY.from_ddy_file(relative_path)

        ddy_json = ddy.to_json()
        reconstructed_ddy = DDY.from_json(ddy_json)
        assert ddy_json == reconstructed_ddy.to_json()

    def test_ddy_from_design_day(self):
        """Test ddy from design day method."""
        relative_path = './tests/ddy/chicago_monthly.ddy'
        ddy = DDY.from_ddy_file(relative_path)
        new_ddy = DDY.from_design_day(ddy.design_days[0])
        assert ddy.location == new_ddy.location
        assert ddy.design_days[0] == new_ddy.design_days[0]

    def test_write_ddy(self):
        """Test write ddy."""
        relative_path = './tests/ddy/chicago.ddy'
        ddy = DDY.from_ddy_file(relative_path)
        new_file_path = './tests/ddy/chicago_edited.ddy'
        ddy.save(new_file_path)

    def test_standard_ddy_properties(self):
        """Test properties of a standard ddy."""
        relative_path = './tests/ddy/tokyo.ddy'

        ddy = DDY.from_ddy_file(relative_path)

        # Test accuracy of import
        assert ddy.location.city == 'TOKYO HYAKURI_JPN Design_Conditions'
        assert ddy.location.latitude == approx(36.18, rel=1e-1)
        assert ddy.location.longitude == approx(140.42, rel=1e-1)
        assert ddy.location.time_zone == 9
        assert ddy.location.elevation == 35

        assert len(ddy.design_days) == 18
        for des_day in ddy.design_days:
            assert hasattr(des_day, 'isDesignDay')
        assert len(ddy.filter_by_keyword('.4%')) == 4
        assert len(ddy.filter_by_keyword('99.6%')) == 3

    def test_monthly_ddy_properties(self):
        """Test properties of a monthly ddy."""
        relative_path = './tests/ddy/chicago_monthly.ddy'
        ddy = DDY.from_ddy_file(relative_path)

        # Test accuracy of import
        assert ddy.location.city == 'Chicago Ohare Intl Ap'
        assert ddy.location.latitude == approx(41.96, rel=1e-1)
        assert ddy.location.longitude == approx(-87.92, rel=1e-1)
        assert ddy.location.time_zone == -6
        assert ddy.location.elevation == 201

        assert len(ddy.design_days) == 12
        for des_day in ddy.design_days:
            assert hasattr(des_day, 'isDesignDay')
            assert des_day.day_type == 'SummerDesignDay'

    def test_design_day_from_properties(self):
        """Test hourly data properties of a standard ddy."""
        location = Location('Test City', '-', 'USA', 34.20, -118.35, -8, 226)
        a_period = AnalysisPeriod(12, 21, 0, 12, 21, 23)
        des_day = DesignDay.from_design_day_properties('Test Day', 'WinterDesignDay',
                                                       location, a_period, 3.9, 0,
                                                       'Wetbulb', 3.9, 98639, 0.8, 330,
                                                       'ASHRAEClearSky', [0])
        assert des_day.location == location
        new_period = des_day.analysis_period
        assert new_period.st_month == a_period.st_month
        assert new_period.st_day == a_period.st_day
        assert new_period.st_hour == a_period.st_hour
        assert new_period.end_month == a_period.end_month
        assert new_period.end_day == a_period.end_day
        assert new_period.end_hour == a_period.end_hour

    def test_design_day_hourly_data(self):
        """Test hourly data properties of a standard ddy."""
        location = Location('Test City', '-', 'USA', 34.20, -118.35, -8, 226)
        a_period = AnalysisPeriod(8, 21, 0, 8, 21, 23)
        des_day = DesignDay.from_design_day_properties('Test Day', 'SummerDesignDay',
                                                       location, a_period, 36.8, 13.2,
                                                       'Wetbulb', 20.5, 98639, 3.9, 170,
                                                       'ASHRAETau', [0.436, 2.106])
        # dry bulb values
        db_data_collect = des_day.hourly_dry_bulb
        assert db_data_collect[5] == approx(23.6, rel=1e-1)
        assert db_data_collect[14] == approx(36.8, rel=1e-1)

        # dew point values
        dpt_data_collect = des_day.hourly_dew_point
        assert dpt_data_collect[0] == approx(11.296, rel=1e-1)
        assert dpt_data_collect[-1] == approx(11.296, rel=1e-1)

        # relative humidity values
        rh_data_collect = des_day.hourly_relative_humidity
        assert rh_data_collect[5] == approx(45.896, rel=1e-1)
        assert rh_data_collect[14] == approx(21.508, rel=1e-1)

        # barometric pressure values
        bp_data_collect = des_day.hourly_barometric_pressure
        assert bp_data_collect[0] == approx(98639, rel=1e-1)
        assert bp_data_collect[-1] == approx(98639, rel=1e-1)

        # wind speed values
        ws_data_collect = des_day.hourly_wind_speed
        assert -0.1 < ws_data_collect[0] - 3.9 < 0.1
        assert -0.1 < ws_data_collect[-1] - 3.9 < 0.1

        # wind direction values
        wd_data_collect = des_day.hourly_wind_direction
        assert wd_data_collect[0] == approx(170, rel=1e-1)
        assert wd_data_collect[-1] == approx(170, rel=1e-1)

        # radiation values
        direct_normal_rad, diffuse_horizontal_rad, global_horizontal_rad = \
            des_day.hourly_solar_radiation
        assert direct_normal_rad[0] == 0
        assert direct_normal_rad[11] == approx(891.46, rel=1e-1)
        assert diffuse_horizontal_rad[0] == 0
        assert diffuse_horizontal_rad[11] == approx(165.32, rel=1e-1)
        assert global_horizontal_rad[0] == 0
        assert global_horizontal_rad[11] == approx(985.05, rel=1e-1)


if __name__ == "__main__":
    unittest.main()



# coding=utf-8

import unittest
import pytest

from ladybug.skymodel import dirint, disc, _get_dirint_coeffs


class SkyModelTestCase(unittest.TestCase):
    """Test for (ladybug/skymodel.py)"""

    def test_dirint(self):
        """Test the accuracy of the dirint model against pvlib results."""
        dirint_result = dirint(
            [1038.62, 254.53], [90 - 10.567, 90 - 72.469],
            [175, 175], [93193., 93193.])
        assert dirint_result[0] == pytest.approx(868.8332, rel=1e-2)
        assert dirint_result[1] == pytest.approx(699.65011, rel=1e-2)

    def test_dirint_tdew(self):
        """Test the accuracy of the dirint model against pvlib results."""
        dirint_result = dirint(
            [1038.62, 254.53], [90 - 10.567, 90 - 72.469],
            [175, 175], [93193., 93193.], temp_dew=[10, 10])

        assert dirint_result[0] == pytest.approx(882.1372, rel=1e-2)
        assert dirint_result[1] == pytest.approx(672.5589, rel=1e-2)

    def test_dirint_no_delta_kt(self):
        """Test the accuracy of the dirint model against pvlib results."""
        dirint_result = dirint(
            [1038.62, 254.53], [90 - 10.567, 90 - 72.469], [175, 175],
            [93193., 93193.], use_delta_kt_prime=False)
        assert dirint_result[0] == pytest.approx(861.857, rel=1e-1)
        assert dirint_result[1] == pytest.approx(670.341, rel=1e-3)

    def test_dirint_coeffs(self):
        """Test the dirint coefficients against pvlib results."""
        coeffs = _get_dirint_coeffs()
        assert coeffs[0][0][0][0] == 0.385230
        assert coeffs[0][1][2][1] == 0.229970
        assert coeffs[3][2][6][3] == 1.032260

    def test_disc(self):
        """Test the accuracy of the disc model against pvlib results."""
        disc_result = disc(1000, 80, 1)
        assert disc_result[0] == pytest.approx(611.40543, rel=1e-2)
        assert disc_result[1] == pytest.approx(0.716088, rel=1e-3)
        assert disc_result[2] == pytest.approx(1.014825, rel=1e-3)

        disc_result = disc(200, 20, 150)
        assert disc_result[0] == pytest.approx(203.0609, rel=1e-2)
        assert disc_result[1] == pytest.approx(0.43897, rel=1e-3)
        assert disc_result[2] == pytest.approx(2.89994, rel=1e-3)

    def test_disc_overirradiance(self):
        """Test overirradiance in the disc model."""
        disc_result = disc(3000, 90, 200)

        assert disc_result[0] == pytest.approx(872.544, rel=1e-2)
        assert disc_result[1] == pytest.approx(1.000, rel=1e-3)
        assert disc_result[2] == pytest.approx(0.999493933, rel=1e-3)


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""Utility functions for converting between humidity metrics."""
from __future__ import division

import math


def saturated_vapor_pressure(t_kelvin):
    """Saturated Vapor Pressure (Pa) at t_kelvin (K).

    This function accounts for the different behaviour above vs. below
    the freezing point of water.

    Note:
        [1] W. Wagner and A. Pru:" The IAPWS Formulation 1995 for the Thermodynamic
        Properties of Ordinary Water Substance for General and Scientific Use ",
        Journal of Physical and Chemical Reference Data,
        June 2002 ,Volume 31, Issue 2, pp. 387535

        [2] Vaisala. (2013) Humidity Conversion Formulas:
        Calculation Formulas for Humidity.
        www.vaisala.com/Vaisala%20Documents/Application%20notes/Humidity_Conversion_Formulas_B210973EN-F.pdf
    """

    if t_kelvin >= 273.15:
        # Calculate saturation vapor pressure above freezing
        sig = 1 - (t_kelvin / 647.096)
        sig_polynomial = (-7.85951783 * sig) + (1.84408259 * sig ** 1.5) + \
            (-11.7866487 * sig ** 3) + (22.6807411 * sig ** 3.5) + \
            (-15.9618719 * sig ** 4) + (1.80122502 * sig ** 7.5)
        crit_temp = 647.096 / t_kelvin
        exponent = crit_temp * sig_polynomial
        p_ws = math.exp(exponent) * 22064000
    else:
        # Calculate saturation vapor pressure below freezing
        theta = t_kelvin / 273.15
        exponent = -13.928169 * (1 - theta ** -1.5) + \
            34.707823 * (1 - theta ** -1.25)
        p_ws = math.exp(exponent) * 611.657
    return p_ws


def humid_ratio_from_db_rh(db_temp, rel_humid, b_press=101325):
    """Humidity Ratio (kg water/kg air) at a db_temp (C),
    rel_humid (%) and Pressure b_press (Pa).

    Note:
        [1] Vaisala. (2013) Humidity Conversion Formulas:
        Calculation Formulas for Humidity.
        www.vaisala.com/Vaisala%20Documents/Application%20notes/Humidity_Conversion_Formulas_B210973EN-F.pdf
    """
    # Find saturation pressure
    p_ws = saturated_vapor_pressure(db_temp + 273.15)
    # Calculate partial pressure
    decrh = rel_humid * 0.01
    p_w = decrh * p_ws
    # Calculate humidity ratio
    press_differ = b_press - p_w
    constant = p_w * 0.621991
    humidity_ratio = constant / press_differ
    return humidity_ratio


def enthalpy_from_db_hr(db_temp, humid_ratio):
    """Enthalpy (kJ/kg) at humid_ratio (kg water/kg air) and at db_temp (C).

    Note:
        [1] Vaisala. (2013) Humidity Conversion Formulas:
        Calculation Formulas for Humidity.
        www.vaisala.com/Vaisala%20Documents/Application%20notes/Humidity_Conversion_Formulas_B210973EN-F.pdf
    """
    enthalpy = ((1.01 + (1.89 * humid_ratio)) * db_temp) + (2500 * humid_ratio)
    return enthalpy if enthalpy >= 0 else 0


def wet_bulb_from_db_rh(db_temp, rh, b_press=101325):
    """Wet Bulb Temperature (C) at db_temp (C),
    Relative Humidity rh (%), and Pressure b_press (Pa).

    Note:
        [1] J. Sullivan and L. D. Sanders. "Method for obtaining wet-bulb temperatures by
        modifying the psychrometric formula." Center for Experiment Design and Data
        Analysis. NOAA - National Oceanic and Atmospheric Administration.
        http://www.srh.noaa.gov/epz/?n=wxcalc_rh
    """
    es = 6.112 * math.e**((17.67 * db_temp) / (db_temp + 243.5))
    e = (es * rh) / 100
    t_w = 0
    increse = 10.0
    previoussign = 1
    e_d = 1
    while math.fabs(e_d) > 0.005:
        e_wg = 6.112 * (math.e**((17.67 * t_w) / (t_w + 243.5)))
        eg = e_wg - (b_press/100) * (db_temp - t_w) * 0.00066 * (1 + (0.00155 * t_w))
        e_d = e - eg
        if e_d == 0:
            break
        else:
            if e_d < 0:
                cursign = -1
                if cursign != previoussign:
                    previoussign = cursign
                    increse = increse / 10
                else:
                    increse = increse
            else:
                cursign = 1
                if cursign != previoussign:
                    previoussign = cursign
                    increse = increse/10
                else:
                    increse = increse
        t_w = t_w + increse * previoussign
    return t_w


def dew_point_from_db_rh(db_temp, rh):
    """Dew Point Temperature (C) at Dry Bulb Temperature db_temp (C) and
    Relative Humidity rh (%).

    Note:
        [1] J. Sullivan and L. D. Sanders. "Method for obtaining wet-bulb temperatures by
        modifying the psychrometric formula." Center for Experiment Design and Data
        Analysis. NOAA - National Oceanic and Atmospheric Administration.
        http://www.srh.noaa.gov/epz/?n=wxcalc_rh
    """
    es = 6.112 * math.e**((17.67 * db_temp) / (db_temp + 243.5))
    e = (es * rh) / 100
    td = (243.5 * math.log(e / 6.112)) / (17.67 - math.log(e / 6.112))
    return td


def rel_humid_from_db_hr(db_temp, humid_ratio, b_press=101325):
    """Relative Humidity (%) at humid_ratio (kg water/kg air), db_temp (C),
    and Pressure b_press (Pa).
    """
    pw = (humid_ratio * 1000 * b_press) / (621.9907 + (humid_ratio * 1000))
    pws = saturated_vapor_pressure(db_temp + 273.15)
    rel_humid = (pw / pws) * 100
    return rel_humid


def rel_humid_from_db_enth(db_temp, enthalpy, b_press=101325):
    """Relative Humidity (%) at db_temp (C), enthalpy (kJ/kg)
    and Pressure b_press (Pa).
    """
    assert enthalpy >= 0, 'enthalpy must be' \
        'greater than 0. Got {}'.format(str(enthalpy))
    hr = (enthalpy - (1.006 * db_temp)) / ((1.84 * db_temp) + 2501)
    rel_humid = rel_humid_from_db_hr(db_temp, hr, b_press)
    return rel_humid


def rel_humid_from_db_dpt(db_temp, dew_pt):
    """Relative Humidity (%) at db_temp (C), and dew_pt (C).
    """
    pws_ta = saturated_vapor_pressure(db_temp + 273.15)
    pws_td = saturated_vapor_pressure(dew_pt + 273.15)
    rh = 100 * (pws_td / pws_ta)
    return rh


def rel_humid_from_db_wb(db_temp, wet_bulb, b_press=101325):
    """Relative Humidity (%) at db_temp(C), wet_bulb (C), and Pressure b_press (Pa).
    """
    # Calculate saturation pressure.
    p_ws = saturated_vapor_pressure(db_temp + 273.15)
    p_ws_wb = saturated_vapor_pressure(wet_bulb + 273.15)
    # calculate partial vapor pressure
    p_w = p_ws_wb - (b_press * 0.000662 * (db_temp - wet_bulb))
    # Calculate the relative humidity.
    rel_humid = (p_w / p_ws) * 100
    return rel_humid


def dew_point_from_db_hr(db_temp, hr, b_press=101325):
    """Dew Point Temperature (C) at Temperature db_temp (C),
    Humidity Ratio hr (kg water/kg air) and Pressure b_press (Pa).
    """
    rh = rel_humid_from_db_hr(db_temp, hr, b_press)
    td = dew_point_from_db_rh(db_temp, rh)
    return td


def dew_point_from_db_enth(db_temp, enthlpy, b_press=101325):
    """Dew Point Temperature (C) at Temperature db_temp (C), enthalpy (kJ/kg)
    and Pressure b_press (Pa).
    """
    rh = rel_humid_from_db_enth(db_temp, enthlpy, b_press)
    td = dew_point_from_db_rh(db_temp, rh)
    return td


def dew_point_from_db_wb(db_temp, wet_bulb, b_press=101325):
    """Dew Point Temperature (C) at Temperature db_temp (C), wet_bulb (C)
    and Pressure b_press (Pa).
    """
    rh = rel_humid_from_db_wb(db_temp, wet_bulb, b_press)
    td = dew_point_from_db_rh(db_temp, rh)
    return td


def db_temp_from_enth_hr(enthalpy, humid_ratio):
    """Dry Bulb Temperature (C) at Enthalpy enthalpy (kJ/kg) and
    humid_ratio (kg water/kg air).
    """
    db_temp = (enthalpy - 2.5 * (humid_ratio * 1000)) / \
        (1.01 + (0.00189 * humid_ratio * 1000))
    return db_temp


def db_temp_from_wb_rh(wet_bulb, rel_humid, b_press=101325):
    """Dry Bulb Temperature (C) and humidity_ratio at at wet_bulb (C),
    rel_humid (%) and Pressure b_press (Pa).

    Formula is only valid for rel_humid == 0 or rel_humid == 100.
    """
    assert rel_humid == 0 or rel_humid == 100, 'formula is only valid for' \
        ' rel_humid == 0 or rel_humid == 100'
    humidity_ratio = humid_ratio_from_db_rh(wet_bulb, rel_humid, b_press)
    hr_saturation = humid_ratio_from_db_rh(wet_bulb, 100, b_press)
    db_temp = wet_bulb + (((hr_saturation - humidity_ratio) * 2260000) / (1005))
    return db_temp, humidity_ratio



# coding=utf-8

import unittest
from pytest import approx
import os

from ladybug.stat import STAT
from ladybug.designday import DDY


class StatTestCase(unittest.TestCase):
    """Test for (ladybug/stat.py)"""

    def test_import_STAT(self):
        """Test import standard stat."""
        relative_path = './tests/stat/chicago.stat'
        abs_path = os.path.abspath(relative_path)

        stat_rel = STAT(relative_path)
        stat = STAT(abs_path)

        # Test imports don't break
        assert stat.file_path == abs_path
        assert stat_rel.file_path == os.path.normpath(relative_path)
        assert isinstance(stat, STAT)

    def test_stat_location(self):
        relative_path = './tests/stat/tokyo.stat'
        stat = STAT(relative_path)

        # Test accuracy of import
        assert len(stat._header) == 10
        assert stat.location.city == 'TOKYO HYAKURI'
        assert stat.location.country == 'JPN'
        assert stat.location.source == 'IWEC Data'
        assert stat.location.station_id == '477150'
        assert stat.location.latitude == approx(36.18, rel=1e-1)
        assert stat.location.longitude == approx(140.42, rel=1e-1)
        assert stat.location.time_zone == 9
        assert stat.location.elevation == 35

        assert stat.monthly_tau_beam == \
            [0.341, 0.392, 0.48, 0.601, 0.654, 0.632, 0.553, 0.546, 0.473,
             0.423, 0.388, 0.338]
        assert stat.monthly_tau_diffuse == \
            [2.214, 1.999, 1.765, 1.546, 1.501, 1.591, 1.811, 1.827, 2.024,
             2.083, 2.105, 2.267]
        assert stat.ashrae_climate_zone == '4A'
        assert stat.koppen_climate_zone == 'Cfa'

    def test_annual_heating_design_days(self):
        relative_path = './tests/stat/chicago.stat'
        stat = STAT(relative_path)

        ann_hdd_96 = stat.annual_heating_design_day_996
        ann_hdd_90 = stat.annual_heating_design_day_990

        assert ann_hdd_96.day_type == 'WinterDesignDay'
        assert ann_hdd_96.name == '99.6% Heating Design Day for Chicago Ohare Intl Ap'
        assert ann_hdd_96.analysis_period.st_month == \
            ann_hdd_96.analysis_period.end_month == \
            ann_hdd_96.sky_condition.month == 1
        assert ann_hdd_96.analysis_period.st_day == \
            ann_hdd_96.analysis_period.end_day == 21
        assert ann_hdd_96.humidity_condition.hum_type == 'Wetbulb'
        assert ann_hdd_96.dry_bulb_condition.dry_bulb_max == \
            ann_hdd_96.humidity_condition.hum_value == -20
        assert ann_hdd_96.dry_bulb_condition.dry_bulb_range == 0
        assert ann_hdd_96.wind_condition.wind_speed == 4.9
        assert ann_hdd_96.wind_condition.wind_direction == 270
        assert ann_hdd_96.sky_condition.solar_model == 'ASHRAEClearSky'
        assert ann_hdd_96.sky_condition.clearness == 0

        assert ann_hdd_90.name == '99.0% Heating Design Day for Chicago Ohare Intl Ap'
        assert ann_hdd_90.dry_bulb_condition.dry_bulb_max == \
            ann_hdd_90.humidity_condition.hum_value == -16.6

    def test_annual_cooling_design_days(self):
        relative_path = './tests/stat/tokyo.stat'
        stat = STAT(relative_path)
        ann_cdd_04 = stat.annual_cooling_design_day_004
        ann_cdd_10 = stat.annual_cooling_design_day_010

        assert ann_cdd_04.day_type == 'SummerDesignDay'
        assert ann_cdd_04.name == '0.4% Cooling Design Day for TOKYO HYAKURI'
        assert ann_cdd_04.analysis_period.st_month == \
            ann_cdd_04.analysis_period.end_month == \
            ann_cdd_04.sky_condition.month == 8
        assert ann_cdd_04.analysis_period.st_day == \
            ann_cdd_04.analysis_period.end_day == 21
        assert ann_cdd_04.humidity_condition.hum_type == 'Wetbulb'
        assert ann_cdd_04.dry_bulb_condition.dry_bulb_max == 32.1
        assert ann_cdd_04.humidity_condition.hum_value == 26.0
        assert ann_cdd_04.dry_bulb_condition.dry_bulb_range == 7.7
        assert ann_cdd_04.wind_condition.wind_speed == 4.8
        assert ann_cdd_04.wind_condition.wind_direction == 210
        assert ann_cdd_04.sky_condition.solar_model == 'ASHRAETau'
        assert ann_cdd_04.sky_condition.tau_b == 0.546
        assert ann_cdd_04.sky_condition.tau_d == 1.827

        assert ann_cdd_10.name == '1.0% Cooling Design Day for TOKYO HYAKURI'
        assert ann_cdd_10.dry_bulb_condition.dry_bulb_max == 30.9
        assert ann_cdd_10.humidity_condition.hum_value == 25.8

    def test_monthly_cooling_design_days(self):
        relative_path = './tests/stat/chicago.stat'
        stat = STAT(relative_path)

        m_ddy_050 = stat.monthly_cooling_design_days_050
        m_ddy_100 = stat.monthly_cooling_design_days_100
        m_ddy_020 = stat.monthly_cooling_design_days_020
        m_ddy_004 = stat.monthly_cooling_design_days_004

        assert len(m_ddy_050) == len(m_ddy_100) == len(m_ddy_020) == \
            len(m_ddy_004) == 12

        ddy_path = './tests/ddy/chicago_monthly.ddy'
        monthly_ddy = DDY(stat.location, m_ddy_050)
        monthly_ddy.save(ddy_path)

    def test_typical_extreme_weeks(self):
        relative_path = './tests/stat/chicago.stat'
        stat = STAT(relative_path)

        extreme_cold = stat.extreme_cold_week
        extreme_hot = stat.extreme_hot_week
        typical_winter = stat.typical_winter_week
        typical_spring = stat.typical_spring_week
        typical_summer = stat.typical_summer_week
        typical_autumn = stat.typical_autumn_week

        assert str(extreme_cold) == '1/27 to 2/2 between 0 and 23 @1'
        assert str(extreme_hot) == '7/13 to 7/19 between 0 and 23 @1'
        assert str(typical_winter) == '12/22 to 12/28 between 0 and 23 @1'
        assert str(typical_spring) == '4/26 to 5/2 between 0 and 23 @1'
        assert str(typical_summer) == '8/24 to 8/30 between 0 and 23 @1'
        assert str(typical_autumn) == '10/27 to 11/2 between 0 and 23 @1'

    def test_json_methods(self):
        """Test JSON serialization methods"""
        relative_path = './tests/stat/chicago.stat'
        stat_obj = STAT(relative_path)

        stat_json = stat_obj.to_json()
        rebuilt_stat = STAT.from_json(stat_json)
        assert stat_json == rebuilt_stat.to_json()


if __name__ == "__main__":
    unittest.main()



# coding=utf-8

import unittest
import pytest
import os

import ladybug.futil as futil


class DataCollectionTestCase(unittest.TestCase):
    """Test for (ladybug/futil.py)"""

    # preparing to test.
    def setUp(self):
        """set up."""

    def tearDown(self):
        """Nothing to tear down as nothing gets written to file."""
        pass

    def test_download_file(self):
        """Test the download file capability"""
        wf_url = "https://www.energyplus.net/weather-download/" \
            "southwest_pacific_wmo_region_5/AUS//AUS_NSW.Sydney.947670_IWEC/all"
        wf_name = os.path.join("./tests/zip", "AUS_NSW.Sydney.947670_IWEC.zip")
        futil.download_file(wf_url, wf_name)
        assert os.path.isfile(wf_name)
        assert 200000 < os.stat(wf_name).st_size < 300000
        os.remove(wf_name)

    def test_unzip_file(self):
        """Test the unzip file capability"""
        folder = "./tests/zip/extracted"
        wf_path = "./tests/zip/test.zip"
        futil.unzip_file(wf_path, folder)
        extracted_epw = os.path.join(folder, "AUS_NSW.Sydney.947670_IWEC.epw")
        assert os.path.isfile(extracted_epw)
        assert 1500000 < os.stat(extracted_epw).st_size < 1600000
        futil.nukedir(folder)

    def test_copy_files_to_folder(self):
        """Test the copy file to folder capability"""
        existing_file = "./tests/ddy/chicago.ddy"
        target_dir = "./tests/zip"
        futil.copy_files_to_folder([existing_file], target_dir)
        final_file = os.path.join(target_dir, "chicago.ddy")
        assert os.path.isfile(final_file)
        assert 20000 < os.stat(final_file).st_size < 30000
        os.remove(final_file)

    def test_csv_to_matrix(self):
        """Test the csv_to_matrix functions."""
        path = './tests/epw/tokyo.epw'
        epw_mtx = futil.csv_to_matrix(path)
        assert len(epw_mtx) == 8768

        with pytest.raises(Exception):
            epw_mtx = futil.csv_to_num_matrix(path)


if __name__ == "__main__":
    unittest.main()



# coding=utf-8

import unittest
from ladybug.location import Location


class LocationTestCase(unittest.TestCase):
    """Test for (ladybug/location.py)"""

    # preparing to test.
    def setUp(self):
        """set up."""
        pass

    def tearDown(self):
        """Nothing to tear down as nothing gets written to file."""
        pass

    def test_default_values(self):
        """Test if the command correctly creates a location."""
        loc = Location()
        assert loc.city == '-'
        assert loc.country == '-'
        assert loc.latitude == 0
        assert loc.longitude == 0
        assert loc.time_zone == 0
        assert loc.station_id is None
        assert loc.source is None

    def test_from__init__(self):
        """Test if the command correctly creates location based on individual values"""
        # This is totally a real place! It's in Wales, check it out!
        city = 'Llanfairpwllgwyngyllgogerychwyrndrobwllllantysiliogogogoch'
        country = 'United Kingdom'
        latitude = 53.2225252
        longitude = -4.2211707
        time_zone = 1
        elevation = 12
        station_id = 'SomeInventedStation'
        source = 'SomeNoneExistentSource'

        loc = Location(city=city, country=country, latitude=latitude,
                       longitude=longitude, time_zone=time_zone,
                       elevation=elevation, station_id=station_id, source=source)

        assert loc.city == city
        assert loc.country == country
        assert loc.latitude == latitude
        assert loc.longitude == longitude
        assert loc.time_zone == time_zone
        assert loc.elevation == elevation
        assert loc.station_id == station_id
        assert loc.source == source
        assert loc.meridian == -15

    def test_from_individual_values(self):
        """Test if the command correctly creates location based on individual values"""
        loc = Location()

        new_latitude = 31.4234953
        new_longitude = 72.9492158
        new_time_zone = 5
        new_elevation = 20

        loc.latitude = new_latitude
        loc.longitude = new_longitude
        loc.time_zone = new_time_zone
        loc.elevation = new_elevation

        assert loc.latitude == new_latitude
        assert loc.longitude == new_longitude
        assert loc.time_zone == new_time_zone
        assert loc.elevation == new_elevation

    def test_from_location(self):
        """Test the from_location() class method"""
        city = 'Tehran'
        country = 'Iran'
        latitude = 36
        longitude = 34
        time_zone = 3.5
        elevation = 54

        loc = Location(city=city, country=country, latitude=latitude,
                       longitude=longitude, time_zone=time_zone,
                       elevation=elevation)

        loc_from_loc = Location.from_location(loc)

        assert loc_from_loc.city == city
        assert loc_from_loc.country == country
        assert loc_from_loc.latitude == latitude
        assert loc_from_loc.longitude == longitude
        assert loc_from_loc.time_zone == time_zone
        assert loc_from_loc.elevation == elevation

    def test_json_methods(self):
        """Test JSON serialization functions"""
        city = 'Tehran'
        country = 'Iran'
        latitude = 36
        longitude = 34
        time_zone = 3.5
        elevation = 54

        loc = Location(city=city, country=country, latitude=latitude,
                       longitude=longitude, time_zone=time_zone,
                       elevation=elevation)

        assert loc.to_json() == {"city": city, "state": '-', "country": country,
                                 "latitude": latitude, "longitude": longitude,
                                 "time_zone": time_zone, "elevation": elevation,
                                 "station_id": None, "source": None}

        loc_from_json = Location.from_json(loc.to_json())

        assert loc_from_json.city == city
        assert loc_from_json.latitude == latitude
        assert loc_from_json.longitude == longitude
        assert loc_from_json.time_zone == time_zone
        assert loc_from_json.elevation == elevation
        assert loc_from_json.country == country

    def test_duplicate(self):
        """Test the duplicate method."""
        city = 'Tehran'
        country = 'Iran'
        latitude = 36
        longitude = 34
        time_zone = 3.5
        elevation = 54

        loc = Location(city=city, country=country, latitude=latitude,
                       longitude=longitude, time_zone=time_zone,
                       elevation=elevation)
        loc_dup = loc.duplicate()

        assert loc.city == loc_dup.city == city
        assert loc.country == loc_dup.country == country
        assert loc.latitude == loc_dup.latitude == latitude
        assert loc.longitude == loc_dup.longitude == longitude
        assert loc.time_zone == loc_dup.time_zone == time_zone
        assert loc.elevation == loc_dup.elevation == elevation


if __name__ == "__main__":
    unittest.main()



# coding=utf-8
"""Ladybug datetime."""
from __future__ import division

from datetime import datetime


class DateTime(datetime):
    """Create Ladybug Date time.

    Args:
        month: A value for month between 1-12 (Defualt: 1).
        day: A value for day between 1-31 (Defualt: 1).
        hour: A value for hour between 0-23 (Defualt: 0).
        minute: A value for month between 0-59 (Defualt: 0).
        leap_year: A boolean to indicate if datetime is for a leap year
            (Default: False).
    """

    __slots__ = ()

    def __new__(cls, month=1, day=1, hour=0, minute=0, leap_year=False):
        """Create Ladybug datetime.

        Args:
            month: A value for month between 1-12 (Defualt: 1).
            day: A value for day between 1-31 (Defualt: 1).
            hour: A value for hour between 0-23 (Defualt: 0).
            minute: A value for month between 0-59 (Defualt: 0).
            leap_year: A boolean to indicate if datetime is for a leap year
                (Default: False).
        """
        year = 2016 if leap_year else 2017
        hour, minute = cls._calculate_hour_and_minute(hour + minute / 60.0)
        try:
            return datetime.__new__(cls, year, month, day, hour, minute)
        except ValueError as e:
            raise ValueError("{}:\n\t({}/{}@{}:{})(m/d@h:m)".format(
                e, month, day, hour, minute
            ))

    @classmethod
    def from_json(cls, data):
        """Creat datetime from a dictionary.

        Args:
            data: {
                'month': A value for month between 1-12. (Defualt: 1)
                'day': A value for day between 1-31. (Defualt: 1)
                'hour': A value for hour between 0-23. (Defualt: 0)
                'minute': A value for month between 0-59. (Defualt: 0)
            }
        """
        if 'month' not in data:
            data['month'] = 1

        if 'day' not in data:
            data['day'] = 1

        if 'hour' not in data:
            data['hour'] = 0

        if 'minute' not in data:
            data['minute'] = 0

        if 'year' not in data:
            data['year'] = 2017

        leap_year = True if int(data['year']) == 2016 else False
        return cls(data['month'], data['day'], data['hour'], data['minute'], leap_year)

    @classmethod
    def from_hoy(cls, hoy, leap_year=False):
        """Create Ladybug Datetime from an hour of the year.

        Args:
            hoy: A float value 0 <= and < 8760
        """
        return cls.from_moy(round(hoy * 60), leap_year)

    @classmethod
    def from_moy(cls, moy, leap_year=False):
        """Create Ladybug Datetime from a minute of the year.

        Args:
            moy: An integer value 0 <= and < 525600
        """
        if not leap_year:
            num_of_minutes_until_month = (0, 44640, 84960, 129600, 172800, 217440,
                                          260640, 305280, 349920, 393120, 437760,
                                          480960, 525600)
        else:
            num_of_minutes_until_month = (0, 44640, 84960 + 1440, 129600 + 1440,
                                          172800 + 1440, 217440 + 1440, 260640 + 1440,
                                          305280 + 1440, 349920 + 1440, 393120 + 1440,
                                          437760 + 1440, 480960 + 1440, 525600 + 1440)
        # find month
        for monthCount in range(12):
            if int(moy) < num_of_minutes_until_month[monthCount + 1]:
                month = monthCount + 1
                break
        try:
            day = int((moy - num_of_minutes_until_month[month - 1]) / (60 * 24)) + 1
        except UnboundLocalError:
            raise ValueError(
                "moy must be positive and smaller than 525600. Invalid input %d" % (moy)
            )
        else:
            hour = int((moy / 60) % 24)
            minute = int(moy % 60)

            return cls(month, day, hour, minute, leap_year)

    @classmethod
    def from_date_time_string(cls, datetime_string, leap_year=False):
        """Create Ladybug DateTime from a DateTime string.

        Usage:

            dt = DateTime.from_date_time_string("31 Dec 12:00")
        """
        dt = datetime.strptime(datetime_string, '%d %b %H:%M')
        return cls(dt.month, dt.day, dt.hour, dt.minute, leap_year)

    @property
    def isDateTime(self):
        """Check if data is ladybug data."""
        return True

    @property
    def doy(self):
        """Calculate day of the year for this date time."""
        return self.timetuple().tm_yday

    @property
    def hoy(self):
        """Calculate hour of the year for this date time."""
        return (self.doy - 1) * 24 + self.float_hour

    @property
    def moy(self):
        """Calculate minute of the year for this date time."""
        return self.int_hoy * 60 + self.minute  # minute of the year

    @property
    def float_hour(self):
        """Get hour and minute as a float value (e.g. 6.25 for 6:15)."""
        return self.hour + self.minute / 60.0

    @property
    def int_hoy(self):
        """Calculate hour of the year for this date time as an integer.

        This output assumes the minute is 0.
        """
        return (self.doy - 1) * 24 + self.hour

    @staticmethod
    def _calculate_hour_and_minute(float_hour):
        """Calculate hour and minutes as integers from a float hour."""
        hour, minute = int(float_hour), int(round((float_hour - int(float_hour)) * 60))
        if minute == 60:
            return hour + 1, 0
        else:
            return hour, minute

    def add_minute(self, minute):
        """Create a new DateTime after the minutes are added.

        Args:
            minute: An integer value for minutes.
        """
        _moy = self.moy + int(minute)
        return self.__class__.from_moy(_moy)

    def sub_minute(self, minute):
        """Create a new DateTime after the minutes are subtracted.

        Args:
            minute: An integer value for the number of minutes.
        """
        return self.add_minute(-minute)

    def add_hour(self, hour):
        """Create a new DateTime from this time + timedelta.

        Args:
            hours: A float value in hours (e.g. .5 = half an hour)
        """
        return self.add_minute(hour * 60)

    def sub_hour(self, hour):
        """Create a new DateTime from this time - timedelta.

        Args:
            hour: A float value in hours (e.g. .5 is half an hour and 2 is two hours).
        """
        return self.add_hour(-hour)

    def to_simple_string(self, separator="_"):
        """Return a simplified string."""
        return self.strftime('%d_%b_%H_%M').replace("_", separator)

    def __str__(self):
        """Return date time as a string."""
        return self.strftime('%d %b %H:%M')

    def to_json(self):
        """Get date time as a dictionary."""
        return {'year': self.year,
                'month': self.month,
                'day': self.day,
                'hour': self.hour,
                'minute': self.minute}

    def ToString(self):
        """Overwrite .NET ToString."""
        return self.__str__()

    def __repr__(self):
        """Return date time as a string."""
        return self.__str__()



# coding=utf-8

import unittest
from ladybug.location import Location
from ladybug.sunpath import Sunpath
from ladybug.dt import DateTime


class SunpathTestCase(unittest.TestCase):
    """Test for (ladybug/sunpath.py)"""

    # preparing to test.
    def setUp(self):
        """set up."""
        pass

    def tearDown(self):
        """Nothing to tear down as nothing gets written to file."""
        pass

    def test_from_location(self):
        sydney = Location('Sydney', 'AUS', latitude=-33.87, longitude=151.22,
                          time_zone=10)
        sunpath = Sunpath.from_location(sydney)
        assert sunpath.latitude == -33.87
        assert sunpath.longitude == 151.22
        assert sunpath.time_zone == 10

    def test_vs_noaa_new_york(self):
        nyc = Location('New_York', 'USA', latitude=40.72, longitude=-74.02,
                       time_zone=-5)
        sp = Sunpath.from_location(nyc)
        sun = sp.calculate_sun(month=9, day=15, hour=11.0)
        assert round(sun.altitude, 2) == 50.35
        assert round(sun.azimuth, 2) == 159.72

    def test_vs_noaa_sydney(self):
        sydney = Location('Sydney', 'AUS', latitude=-33.87, longitude=151.22,
                          time_zone=10)
        sp = Sunpath.from_location(sydney)
        sun = sp.calculate_sun(month=11, day=15, hour=11.0)
        assert round(sun.altitude, 2) == 72.26
        assert round(sun.azimuth, 2) == 32.37

    def test_sunrise_sunset(self):
        pass

    def test_solar_hour(self):
        pass

    def test_daylight_saving(self):
        nyc = Location('New_York', 'USA', latitude=40.72, longitude=-74.02,
                       time_zone=-5)
        sp = Sunpath.from_location(nyc)
        dt1 = DateTime(6, 21, 12, 0)
        dt2 = DateTime(12, 21, 12, 0)
        # TODO(mostapha): This is not implemented yet
        # assert sp.is_daylight_saving_hour(dt1) is True
        assert sp.is_daylight_saving_hour(dt1) is False
        assert sp.is_daylight_saving_hour(dt2) is False

    def test_leap_year(self):
        nyc = Location('New_York', 'USA', latitude=40.72, longitude=-74.02,
                       time_zone=-5)
        sp = Sunpath.from_location(nyc)
        sp.is_leap_year = True
        sun = sp.calculate_sun(month=2, day=29, hour=11.0)
        assert sun.datetime == DateTime(2, 29, 11, leap_year=True)
        assert sun.datetime.year == 2016
        assert sun.datetime.month == 2
        assert sun.datetime.day == 29
        assert sun.datetime.hour == 11


if __name__ == "__main__":
    unittest.main()
